/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File    : USBH_Printer_Start.c
Purpose : Sample program for embOS & USBH USB host stack.
          Demonstrates use of the USBH stack.
--------- END-OF-HEADER --------------------------------------------*/


#include <stdio.h>
#include "RTOS.h"
#include "BSP.h"
#include "USBH.h"
#include "USBH_PrinterClass.h"

#include "Main.h"

/*********************************************************************
*
*       Local defines, configurable
*
**********************************************************************
*/
enum {
  TASK_PRIO_APP = 150,
  TASK_PRIO_USBH_MAIN,
  TASK_PRIO_USBH_ISR
};

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static OS_STACKPTR int     _StackMain[1536/sizeof(int)];
static OS_TASK             _TCBMain;

static OS_STACKPTR int     _StackIsr[1024/sizeof(int)];
static OS_TASK             _TCBIsr;
static volatile char       _PrinterReady;
static U8                  _DevIndex;


/*********************************************************************
*
*       Static Code
*
**********************************************************************
*/

static const char* _FindChar(const char* s, char c) { 
  while (*s != c) {
    if (*s == '\0') {
      break;
    }
    s++;
  }
  return s;
}

static int _StrStartsWith(const char* s1, const char* s2, int length) {
  int i = 0;
  while(i < length) {
    if(*(s1+i) == *(s2+i) && *(s1+i) != '\0') {
      i++;
      continue;
    }
    return 0;
  }
  return 1;
}

static const char* _ProcessDeviceInfo(const char* s) {
  const char* sInfo = _FindChar(s, ':') + 1;

  if(_StrStartsWith(s, "MFG", 3)) {
    int i;
    for(i = 0; i < PRINTER_MANU_LEN-1 && *sInfo != ';'; i++) {
      MAIN_acPrinterManu[i] = *sInfo++;
    }
    MAIN_acPrinterManu[i] = 0;
  } else if(_StrStartsWith(s, "MDL", 3)) {
    int i;
    for(i = 0; i < PRINTER_MANU_LEN-1 && *sInfo != ';'; i++) {
      MAIN_acPrinterModel[i] = *sInfo++;
    }
    MAIN_acPrinterModel[i] = 0;
  } else {
    sInfo = _FindChar(s, ';');
  }
  return sInfo + 1; 
}

/*********************************************************************
*
*       _OnPrinterReady
*
*  Function description
*    Called by the application task (MainTask) when an printer device is
*    plugged in.
*    It reads and displays information about the printer and
*    prints out "Hello world".
*/
USBH_PRINTER_HANDLE hPrinter;
char         ac[20];

static void _OnPrinterReady(void) {

  SEGGER_snprintf(ac, sizeof(ac), "prt%.3d", _DevIndex);
  hPrinter = USBH_PRINTER_Open(ac);
  if (hPrinter) {
    USBH_PRINTER_GetDeviceId(hPrinter, MAIN_acDeviceId, sizeof(MAIN_acDeviceId));
    USBH_PRINTER_GetPortStatus(hPrinter, &MAIN_PortStatus);

    const char* pos = (char*)&MAIN_acDeviceId[2];

    while(*pos != '\0') {
      pos = _ProcessDeviceInfo(pos);
    }

    USBH_Logf_Application("MANUFACTURER = %s", MAIN_acPrinterManu);
    USBH_Logf_Application("MODEL = %s", MAIN_acPrinterModel);
    USBH_Logf_Application("Device Id  = %s", &MAIN_acDeviceId[2]);
    USBH_Logf_Application("PortStatus = 0x%x ->NoError=%d, Select/OnLine=%d, PaperEmpty=%d", MAIN_PortStatus, (MAIN_PortStatus & (1 << 3)) >> 3, (MAIN_PortStatus & (1 << 4)) >> 4, (MAIN_PortStatus & (1 << 5)) >> 5);
    

    //USBH_Logf_Application("Printing to printer");
    //USBH_PRINTER_Write(hPrinter, test2_file, TEST2_SIZE);
    //USBH_Logf_Application("Printing completed");
    USBH_PRINTER_Close(hPrinter);
  }
  //
  // Wait until Printer is removed.
  //
  while (_PrinterReady) {
    OS_Delay(100);
  }
}

void ConnectToPrinter(void) {
  hPrinter = USBH_PRINTER_Open(ac);
}

void DisconnectFromPrinter(void) {
   USBH_PRINTER_Close(hPrinter);
}

USBH_STATUS WriteToPrinter(const U8 *pData, unsigned NumBytes) {
  return USBH_PRINTER_Write(hPrinter, pData, NumBytes);
}

/*********************************************************************
*
*       _cbOnAddRemoveDevice
*
*  Function description
*    Callback, called when a device is added or removed.
*    Call in the context of the USBH_Task.
*    The functionality in this routine should not block
*/
static void _cbOnAddRemoveDevice(void * pContext, U8 DevIndex, USBH_DEVICE_EVENT Event) {
  pContext = pContext;
  switch (Event) {
  case USBH_DEVICE_EVENT_ADD:
    USBH_Logf_Application("**** Device added [%d]\n", DevIndex);
    _PrinterReady = 1;
    break;
  case USBH_DEVICE_EVENT_REMOVE:
    USBH_Logf_Application("**** Device removed [%d]\n", DevIndex);
    _PrinterReady = 0;
    break;
  default:;   // Should never happen
  }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       MainTask
*/
#ifdef __cplusplus
extern "C" {     /* Make sure we have C-declarations in C++ programs */
#endif
void PrintTask(void);
#ifdef __cplusplus
}
#endif
void PrintTask(void) {
  USBH_Init();
  OS_SetPriority(OS_GetTaskID(), TASK_PRIO_APP);                                       // This task has the lowest prio for real-time application.
                                                                                       // Tasks using emUSB-Host API should always have a lower priority than emUSB-Host main and ISR tasks.
  OS_CREATETASK(&_TCBMain, "USBH_Task", USBH_Task, TASK_PRIO_USBH_MAIN, _StackMain);   // Start USBH main task
  OS_CREATETASK(&_TCBIsr, "USBH_isr", USBH_ISRTask, TASK_PRIO_USBH_ISR, _StackIsr);    // Start USBH ISR task
  USBH_PRINTER_Init();
  USBH_PRINTER_RegisterNotification(_cbOnAddRemoveDevice, NULL);
  while (1) {
    BSP_ToggleLED(1);
    OS_Delay(100);
    if (_PrinterReady) {
      _OnPrinterReady();
    }
  }
}




/*************************** End of file ****************************/
