/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File    : Webserver_DynContent.c
Purpose : Dynamic content the embOS/IP Web server.
--------- END-OF-HEADER --------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#ifndef _WIN32
#include "RTOS.h"
#include "BSP.h"
#include "IP.h"
#else
#include <Windows.h>
#include <time.h>
#endif

#include "Main.h"

#include "IP_Webserver.h"
#include "WEBS_Conf.h"        // Stack size depends on configuration

/*********************************************************************
*
*       Local defines, configurable
*
**********************************************************************
*/

#define MAX_CONNECTION_INFO      10
#define NUM_STOCKS               26
#define NUM_VALUES               30

//
// UPnP
//
#define UPNP_FRIENDLY_NAME     "SEGGER UPnP Demo"
#define UPNP_MANUFACTURER      "SEGGER Microcontroller GmbH"
#define UPNP_MANUFACTURER_URL  "http://www.segger.com"
#define UPNP_MODEL_DESC        "SEGGER Web server with UPnP"
#define UPNP_MODEL_NAME        "SEGGER UPnP Demo"
#define UPNP_MODEL_URL         "http://www.segger.com/emweb"

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

//
//lint -esym(534, IP_WEBS_SendString, IP_WEBS_SendFormattedString, IP_PrintIPAddr, IP_WEBS_SendUnsigned, IP_WEBS_SendMem) don't check output of some functions.
//lint -esym(534, IP_WEBS_SendHeaderEx, IP_WEBS_SendHeader, IP_WEBS_METHOD_CopyData, SEGGER_snprintf, getsockname, IP_SOCK_getsockname)
//

/******************************************************************************************************************************************
*
*       CGI sample functions
*
*******************************************************************************************************************************************
*/

/*********************************************************************
*
*       _callback_CGI_UploadFile
*/
static void _callback_CGI_UploadFile(WEBS_OUTPUT* pOutput, const char* sParameters) {
        int   r;
  const char* pFileName;
        int   FileNameLen;
  const char* pState;       // Can be 0: Upload failed; 1: Upload succeeded; Therefore we do not need to know the length, it will always be 1.

  r  = IP_WEBS_GetParaValuePtr(sParameters, 0, NULL, 0, &pFileName, &FileNameLen);
  r |= IP_WEBS_GetParaValuePtr(sParameters, 1, NULL, 0, &pState   , NULL);
  if (r == 0) {
    if (*pState == '1') {
      IP_WEBS_SendString(pOutput, "OK");
    } else {
      IP_WEBS_SendString(pOutput, "ERROR");
    }
  } else {
    IP_WEBS_SendString(pOutput, "ERROR");  
  }
}

/*********************************************************************
*
*       _GetTimeDate
*
*  Function description:
*    Returns current time and date.
*
*    Bit 0-4:   2-second count (0-29)
*    Bit 5-10:  Minutes (0-59)
*    Bit 11-15: Hours (0-23)
*    Bit 16-20: Day of month (1-31)
*    Bit 21-24: Month of year (1-12)
*    Bit 25-31: Count of years from 1980 (0-127)
*
*  Note:
*    This is a sample implementation for a clock-less system.
*    It always returns 01 Jan 2009 00:00:00 GMT
*/
static U32 _GetTimeDate(void) {
  U32 r;
  U16 Sec, Min, Hour;
  U16 Day, Month, Year;

  Sec   = 0;        // 0 based.  Valid range: 0..59
  Min   = 0;        // 0 based.  Valid range: 0..59
  Hour  = 0;        // 0 based.  Valid range: 0..23
  Day   = 1;        // 1 based.    Means that 1 is 1. Valid range is 1..31 (depending on month)
  Month = 1;        // 1 based.    Means that January is 1. Valid range is 1..12.
  Year  = 29;       // 1980 based. Means that 2008 would be 28.
  r   = Sec / 2 + (Min << 5) + (Hour  << 11);
  r  |= (U32)(Day + (Month << 5) + (Year  << 9)) << 16;
  return r;
}

/*********************************************************************
*
*       _pfGetFileInfo
*/
static void _pfGetFileInfo(const char *sFilename, IP_WEBS_FILE_INFO *pFileInfo){
  int v;

  //
  // .cgi files are virtual, everything else is not
  //
  v = IP_WEBS_CompareFilenameExt(sFilename, ".cgi");
  pFileInfo->IsVirtual = v ? 0 : 1;
  //
  // .html files contain dynamic content, everything else is not
  //
  v = IP_WEBS_CompareFilenameExt(sFilename, ".html");
  pFileInfo->AllowDynContent = v ? 0 : 1;
  //
  // .htm files contain dynamic content, everything else is not
  //
  v = IP_WEBS_CompareFilenameExt(sFilename, ".htm");
  pFileInfo->AllowDynContent = v ? 0 : 1;
  //
  // If file is a virtual file or includes dynamic content,
  // get current time and date stamp as file time
  //
  pFileInfo->DateLastMod = _GetTimeDate();
  if (pFileInfo->IsVirtual || pFileInfo->AllowDynContent) {
    //
    // Set expiration time and date
    //
    pFileInfo->DateExp = _GetTimeDate(); // If "Expires" HTTP header field should be transmitted, set expiration date.
  } else {
    pFileInfo->DateExp = 0xEE210000;     // Expiration far away (01 Jan 2099) if content is static
  }
}

/*********************************************************************
*
*       _CB_HandleParameter
*/
static void _CB_HandleParameter(WEBS_OUTPUT *pOutput, const char *sPara, const char *sValue) {
  WEBS_USE_PARA(pOutput);
  WEBS_USE_PARA(sPara);
  WEBS_USE_PARA(sValue);
}

/*********************************************************************
*
*       _aAccessControl_Basic
*/
static const WEBS_ACCESS_CONTROL _aAccessControl_Basic[] = {
  { NULL    , NULL                     , NULL       , NULL }
};

/*********************************************************************
*
*       _aVFiles
*
*  Function description
*    Defines all virtual files used by the web server.
*/
static const WEBS_VFILES _aVFiles[]  = {
  {"Upload.cgi",         _callback_CGI_UploadFile        },   // Called from Upl.htm and Upl_AJAX.htm
  { NULL, NULL }
};

/*********************************************************************
*
*       _callback_DefaultHandler
*/
static void _callback_DefaultHandler(WEBS_OUTPUT* pOutput, const char* sPara, const char* sValue) {
  WEBS_USE_PARA(sValue);

  IP_WEBS_SendString(pOutput, "<h1 style=\"color:red\">Unknown CGI ");
  IP_WEBS_SendString(pOutput, sPara);
  IP_WEBS_SendString(pOutput, " - Ignored</h1>");
}

/*********************************************************************
*
*       _aCGI
*
*  Function description
*    CGI table, defining callback routines for dynamic content (SSI).
*/


static void _callback_PrinterTitle(WEBS_OUTPUT* pOutput, const char* sPara, const char *sValue) {
  WEBS_USE_PARA(sValue);
  WEBS_USE_PARA(sPara);

  IP_WEBS_SendString(pOutput, MAIN_acPrinterModel);
}
static void _callback_DeviceName(WEBS_OUTPUT* pOutput, const char* sPara, const char* sValue) {
  WEBS_USE_PARA(sValue);
  WEBS_USE_PARA(sPara);

  IP_WEBS_SendString(pOutput, MAIN_acPrinterModel);
}
static void _callback_DeviceManufacturer(WEBS_OUTPUT* pOutput, const char* sPara, const char* sValue) {
  WEBS_USE_PARA(sValue);
  WEBS_USE_PARA(sPara);

  IP_WEBS_SendString(pOutput, MAIN_acPrinterManu);
}

static void _callback_StatusPower(WEBS_OUTPUT* pOutput, const char* sPara, const char* sValue) {
  WEBS_USE_PARA(sPara);
  WEBS_USE_PARA(sValue);

//  IP_WEBS_SendString(pOutput, (MAIN_PortStatus & (1 << 4)) ? "True" : "False");
  IP_WEBS_SendString(pOutput, (MAIN_PortStatus & (1 << 4)) ? "False" : "True");
}
static void _callback_PaperNotEmpty(WEBS_OUTPUT* pOutput, const char* sPara, const char* sValue) {
  WEBS_USE_PARA(sPara);
  WEBS_USE_PARA(sValue);

  IP_WEBS_SendString(pOutput, !(MAIN_PortStatus & (1 << 5)) ? "True" : "False");
}

static const WEBS_CGI _aCGI[] = {
  {"PrinterTitle"           , _callback_PrinterTitle           },
  {"DeviceName"             , _callback_DeviceName             },
  {"DeviceManufacturer"     , _callback_DeviceManufacturer     },
  {"StatusPower"            , _callback_StatusPower            },
  {"RemainingSites"         , _callback_PaperNotEmpty          },
  {NULL                     , _callback_DefaultHandler         }
};

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       WebsSample_Application
*
*  Function description
*    Application data table, defines all application specifics used
*    by the web server.
*/
const WEBS_APPLICATION WebsSample_Application = {
  &_aCGI[0],
  (WEBS_ACCESS_CONTROL*)&_aAccessControl_Basic[0],
  _CB_HandleParameter,
  &_aVFiles[0]
};

/*********************************************************************
*
*       IP_WEBS_X_SampleConfig()
*
*  Function description
*    Helper function used in web server samples. It is used as central
*    point to configure sample functionality like adding a REST
*    resource or switching between Basic and Digest authentication.
*/
void IP_WEBS_X_SampleConfig(void) {
  IP_WEBS_SetFileInfoCallback(_pfGetFileInfo);
}

/*************************** End of file ****************************/
