/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
-------------------------- END-OF-HEADER -----------------------------

File    : FS_NAND_Dump.c
Purpose : Saves the entire contents of a NAND flash to an emFile volume.
*/

/*********************************************************************
*
*       #include Section
*
**********************************************************************
*/
#include <string.h>
#include <stdio.h>
#include "FS.h"

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
#define BUFFER_SIZE       (20 * 1024)
#define VOLUME_NAME       "mmc:0:"
#define FILE_NAME         VOLUME_NAME"\\nanddump.bin"

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static char       _ac[128];
static U8         _aBuffer[BUFFER_SIZE];
static U8         _Unit = 0;
static unsigned   _BytesPerPage;
static unsigned   _SectorsPerPage;
static FS_FILE  * _pDumpFile;

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

/*********************************************************************
*
*       _OpenDumpFile
*/
static int _OpenDumpFile(const char * sFileName, U32 NumBytes) {
  _pDumpFile = FS_FOpen(sFileName, "wb");
  if (_pDumpFile) {
    //
    // Pre-allocate the dump file.
    //
    FS_FSeek(_pDumpFile, (I32)NumBytes, FS_SEEK_SET);
    FS_SetEndOfFile(_pDumpFile);
    FS_FSeek(_pDumpFile, 0, FS_SEEK_SET);
  }
  if (_pDumpFile == NULL) {
    return 1;
  }
  return 0;
}

/*********************************************************************
*
*       _WriteToDumpFile
*/
static int _WriteToDumpFile(U8 * pBuffer, U32 NumBytes) {
  U32 NumBytesWritten;

  NumBytesWritten = FS_FWrite(pBuffer, 1, NumBytes, _pDumpFile);
  if (NumBytesWritten != NumBytes) {
    return 1;
  }
  return 0;
}

/*********************************************************************
*
*       _CloseDumpFile
*/
static int _CloseDumpFile(void) {
  int r;

  r = FS_FClose(_pDumpFile);
  return r;
}

/*********************************************************************
*
*       _UnmountDumpVolume
*/
static void _UnmountDumpVolume(void) {
  FS_Unmount(VOLUME_NAME);
}

/*********************************************************************
*
*       _ReadOnePage
*/
static void _ReadOnePage(U32 PageNo, U8 * pBuffer) {
  U32        iSector;
  U32        SectorIndex;
  U8       * pData;
  U8       * pSpare;
  unsigned   NumBytesData;
  unsigned   NumBytesSpare;
  unsigned   BytesPerSector;

  SectorIndex    = PageNo * _SectorsPerPage;
  BytesPerSector = _BytesPerPage / _SectorsPerPage;
  pData          = pBuffer;
  pSpare         = pBuffer + BytesPerSector * _SectorsPerPage;
  NumBytesData   = BytesPerSector;
  NumBytesSpare  = BytesPerSector >> 5;
  for (iSector = 0; iSector < _SectorsPerPage; ++iSector) {
    FS_NAND_ReadPhySector(_Unit, SectorIndex, pData, &NumBytesData, pSpare, &NumBytesSpare);
    ++SectorIndex;
    pData  += NumBytesData;
    pSpare += NumBytesSpare;
  }
}

/*********************************************************************
*
*       _ReadPages
*/
static U32 _ReadPages(U32 PageNo, unsigned NumPages, U8 * pBuffer) {
  U32 NumBytesInPage;
  U32 NumBytes;

  NumBytes       = 0;
  NumBytesInPage = _BytesPerPage + (_BytesPerPage >> 5);
  do {
    _ReadOnePage(PageNo++, pBuffer);
    pBuffer  += NumBytesInPage;
    NumBytes += NumBytesInPage;
  } while (--NumPages);
  return NumBytes;
}

/*********************************************************************
*
*       _DumpFlash
*/
static void _DumpFlash(U8 Unit) {
  unsigned   PageNo;
  unsigned   NumPages;
  unsigned   NumPagesAtOnce;
  U32        NumBytesToWrite;
  U32        BufferSizeMin;
  U32        BytesPerSpareArea;
  unsigned   NumPagesInBuffer;
  U32        NumBytes;
  int        r;
  FS_NAND_DISK_INFO   DiskInfo;

  //
  // Get and show information about the NAND flash.
  //
  FS_NAND_GetDiskInfo(Unit, &DiskInfo);
  BytesPerSpareArea = DiskInfo.BytesPerPage >> 5;
  sprintf(_ac, "NumBlocks:         %lu\n", DiskInfo.NumPhyBlocks);
  FS_X_Log(_ac);
  sprintf(_ac, "PagesPerBlock:     %lu\n", DiskInfo.NumPagesPerBlock);
  FS_X_Log(_ac);
  sprintf(_ac, "BytesPerPage:      %lu\n", DiskInfo.BytesPerPage);
  FS_X_Log(_ac);
  sprintf(_ac, "BytesPerSpareArea: %lu\n", BytesPerSpareArea);
  FS_X_Log(_ac);

  _BytesPerPage    = DiskInfo.BytesPerPage;
  _SectorsPerPage  = DiskInfo.BytesPerPage / DiskInfo.BytesPerSector;
  NumPages         = DiskInfo.NumPhyBlocks * DiskInfo.NumPagesPerBlock;
  BufferSizeMin    = _BytesPerPage + (_BytesPerPage >> 5);
  NumPagesInBuffer = sizeof(_aBuffer) / BufferSizeMin;
  if (NumPagesInBuffer == 0) {
    sprintf(_ac, "Buffer too small. Must be at least %lu bytes in size.\n", BufferSizeMin);
    FS_X_Log(_ac);
    return;
  }
  //
  // Open the file which the contents of the NAND flash.
  //
  NumBytes = NumPages * (_BytesPerPage + BytesPerSpareArea);
  r = _OpenDumpFile(FILE_NAME, NumBytes);
  if (r) {
    sprintf(_ac, "Could not open dump file %s.\n", FILE_NAME);
    FS_X_Log(_ac);
    return;
  }
  sprintf(_ac, "Dumping %u pages", NumPages);
  FS_X_Log(_ac);
  //
  // Read pages from NAND flash and write them to dump file.
  //
  PageNo = 0;
  do {
    NumPagesAtOnce  = SEGGER_MIN(NumPagesInBuffer, NumPages);
    NumBytesToWrite = _ReadPages(PageNo, NumPagesAtOnce, _aBuffer);
    r = _WriteToDumpFile(_aBuffer, NumBytesToWrite);
    if (r) {
      sprintf(_ac, "Could not write pages %u-%u. Dump failed.\n", PageNo, PageNo + NumPagesAtOnce - 1);
      FS_X_Log(_ac);
      break;
    }
    FS_X_Log(".");
    PageNo   += NumPagesAtOnce;
    NumPages -= NumPagesAtOnce;
  } while (NumPages);
  //
  // Close the dump file.
  //
  _CloseDumpFile();
}

/*********************************************************************
*
*             Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       MainTask
*/
#ifdef __cplusplus
extern "C" {     /* Make sure we have C-declarations in C++ programs */
#endif
void MainTask(void);
#ifdef __cplusplus
}
#endif
void MainTask(void) {
  FS_X_Log("Start\n");
  FS_Init();
  _DumpFlash(0);
  _UnmountDumpVolume();
  FS_X_Log("Finished\n");
  while (1) {
    ;
  }
}

