/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File    : USB_MSD_MTP_FS_Start.c
Purpose : This sample demonstrates the combination of MSD and MTP into
          a single interface. The device enumerates as MSD or as MTP
          depending on the host's operating system.
          This is especially useful when using macOS as a host, because
          it does not support MTP natively.
--------  END-OF-HEADER  ---------------------------------------------
*/


#include <stdio.h>
#include <string.h>
#include "USB.h"
#include "USB_MSD_MTP.h"
#include "USB_MSD.h"
#include "USB_MTP.h"
#include "FS.h"
#include "BSP.h"

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
#define BUFFER_SIZE       8192
#define NUM_BYTES_OBJECT_LIST     (4 * 1024)
#define NUM_BYTES_DATA_BUFFER     (4 * USB_MAX_PACKET_SIZE)
#define NUM_BYTES_RECEIVE_BUFFER  (2 * USB_MAX_PACKET_SIZE)

/*********************************************************************
*
*       Static const data
*
**********************************************************************
*/
//
//  Information that is used during enumeration.
//
static const USB_DEVICE_INFO _DeviceInfo = {
  0x8765,         // VendorId
  0x1002,         // ProductId
  "Vendor",       // VendorName
  "MTP device",   // ProductName
  "000013245678"  // SerialNumber. Should be 12 character or more for compliance with Mass Storage Device For Bootability spec.
};
//
// String information used when inquiring the volume 0.
//
static const USB_MSD_LUN_INFO _Lun0Info = {
  "Vendor",     // MSD VendorName
  "MSD Volume", // MSD ProductName
  "1.00",       // MSD ProductVer
  "134657890"   // MSD SerialNo
};

static USB_MTP_INFO _MTPInfo = {
  "Vendor",         // Manufacturer
  "Storage device", // Model
  "1.0",            // DeviceVersion
  "0123456789ABCDEF0123456789ABCDEF" // SerialNumber. It must be exactly 32 characters long.
};

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static U32 _aSectorBuffer[BUFFER_SIZE / 4];     // Used as sector buffer in order to do read/write sector bursts (~8 sectors at once)
static U8  _acReceiveBuffer[NUM_BYTES_RECEIVE_BUFFER];    // Receive buffer for the USB device
static U32 _aDataBuffer[NUM_BYTES_DATA_BUFFER / 4];       // Send/receive buffer for MTP transactions
static U32 _aObjectList[NUM_BYTES_OBJECT_LIST / 4];       // Memory pool for the MTP object list


/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

/*********************************************************************
*
*       _FSTest
*/
static void _FSTest(void) {
  FS_FILE    * pFile;
  unsigned     NumBytes;
  const char * sInfo = "This sample is based on the SEGGER emUSB software with the MTP component.\r\nFor further information please visit: www.segger.com\r\n";
  unsigned     NumVolumes;
  unsigned     i;
  char         acVolumeName[32];

  NumBytes   = strlen(sInfo);
  NumVolumes = FS_GetNumVolumes();
  for (i = 0; i < NumVolumes; i++) {
    FS_GetVolumeName(i, &acVolumeName[0], sizeof(acVolumeName));
    if (FS_IsLLFormatted(acVolumeName) == 0) {
      FS_X_Log("Low-level format\n");
      FS_FormatLow(acVolumeName);         // Erase and low-level format the volume.
    }
    if (FS_IsHLFormatted(acVolumeName) == 0) {
      FS_X_Log("High-level formatting\n");
      FS_Format(acVolumeName, NULL);      // High-level format the volume.
    }
    strcat(acVolumeName, "\\Readme.txt");
    pFile = FS_FOpen(acVolumeName, "w");
    if (pFile) {
      FS_Write(pFile, sInfo, NumBytes);
      FS_FClose(pFile);
    }
  }
}

/*********************************************************************
*
*       _AddMSDMTP
*
*  Function description
*
*/
static void _AddMSDMTP(void) {
  USB_MSD_INIT_DATA MSDInitData;
  USB_MSD_INST_DATA MSDInstData;
  USB_MTP_INIT_DATA MTPInitData;
  USB_MTP_INST_DATA MTPInstData;
  //
  // Set the endpoints in the MSD init data to the same Endpoints as MTP.
  //
  MTPInitData.EPIn                = USBD_AddEP(1, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, NULL, 0);
  MSDInitData.EPIn                = MTPInitData.EPIn;
  MTPInitData.EPOut               = USBD_AddEP(0, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, _acReceiveBuffer, sizeof(_acReceiveBuffer));
  MSDInitData.EPOut               = MTPInitData.EPOut;
  MTPInitData.EPInt               = USBD_AddEP(1, USB_TRANSFER_TYPE_INT, 10, NULL, 0);
  MTPInitData.pObjectList         = _aObjectList;
  MTPInitData.NumBytesObjectList  = sizeof(_aObjectList);
  MTPInitData.pDataBuffer         = _aDataBuffer;
  MTPInitData.NumBytesDataBuffer  = sizeof(_aDataBuffer);
  MTPInitData.pMTPInfo            = &_MTPInfo;
  USBD_SetDeviceInfo(&_DeviceInfo);
  USBD_MSD_MTP_Add(&MSDInitData, &MTPInitData);
  //
  // Add logical unit 0 to MSD
  //
  memset(&MSDInstData, 0,  sizeof(MSDInstData));
  MSDInstData.pAPI                    = &USB_MSD_StorageByName;    // s. Note (1)
  MSDInstData.DriverData.pStart       = (void *)"";
  MSDInstData.DriverData.pSectorBuffer   = _aSectorBuffer;
  MSDInstData.DriverData.NumBytes4Buffer = sizeof(_aSectorBuffer);
  MSDInstData.pLunInfo = &_Lun0Info;
  USBD_MSD_AddUnit(&MSDInstData);
  //
  // Add a storage driver to MTP component.
  //
  MTPInstData.pAPI                = &USB_MTP_StorageFS;
  MTPInstData.sDescription        = "MTP volume";
  MTPInstData.sVolumeId           = "0123456789";
  MTPInstData.DriverData.pRootDir = "";
  USBD_MTP_AddStorage(&MTPInstData);
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       MainTask
*
*/
#ifdef __cplusplus
extern "C" {     /* Make sure we have C-declarations in C++ programs */
#endif
void MainTask(void);
#ifdef __cplusplus
}
#endif
void MainTask(void) {
  USBD_Init();
  FS_Init();
  FS_FAT_SupportLFN();
  _FSTest();
  _AddMSDMTP();
  USBD_Start();
  while (1) {
    //
    // Wait for configuration
    //
    while ((USBD_GetState() & (USB_STAT_CONFIGURED | USB_STAT_SUSPENDED)) != USB_STAT_CONFIGURED) {
      BSP_ToggleLED(0);
      USB_OS_Delay(50);
    }
    BSP_SetLED(0);
    USBD_MSD_MTP_Task();
  }
}

/**************************** end of file ***************************/
