/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
-------------------------- END-OF-HEADER -----------------------------
*/

#include "BSP.h"
#include "RTOS.h"
#include "IP.h"
#include "USB_ECM.h"
#include "USB_Driver_embOS_IP.h"

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
#define ALLOC_SIZE                 0x6000                      // Size of memory dedicated to the stack in bytes
#define DRIVER                     &USB_IP_Driver              // Driver used for target.
#define SERVER_IP_ADDR             IP_BYTES2ADDR(10,    0,   0,  10)
#define DHCPS_START_IP_ADDR        IP_BYTES2ADDR(10,    0,   0,  11)
#define SUBNET_MASK                IP_BYTES2ADDR(255, 255,   0,   0)
#define DNS_SERVER_IP_ADDR         SERVER_IP_ADDR

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/
#ifdef __ICCARM__          // IAR ARM compiler
  #define PRAGMA(x)         _Pragma(#x)
  #define SECTION(x)                  PRAGMA(location=x)
  #define SECTION_NO_INIT(x)          PRAGMA(location=x)   __no_init
  #define SECTION_NO_INIT(x)          PRAGMA(location=x)   __no_init
  #define SECTION_NO_INIT_STATIC(x)   PRAGMA(location=x)   static __no_init
  #define ___WEAK                     __weak
#elif (defined __CC_ARM)   // Keil/ARM compiler
  #define SECTION(x)                  __attribute__ ((section (x)))
  #define SECTION_NO_INIT(x)          __attribute__ ((section (x), zero_init))
  #define SECTION_NO_INIT_STATIC(x)   __attribute__ ((section (x), zero_init)) static
  #define ___WEAK                     __attribute__ ((weak))
#elif (defined __GNUC__)   // GNU ARM compiler
  #define SECTION(x)                  __attribute__ ((section (x)))
  #define SECTION_NO_INIT(x)          __attribute__ ((section (x)))
  #define SECTION_NO_INIT_STATIC(x)   __attribute__ ((section (x))) static
  #define ___WEAK                     __attribute__ ((weak))
#else
  #define SECTION(x)
  #define SECTION_NO_INIT(x)
  #define SECTION_NO_INIT_STATIC(x)   static
  #define ___WEAK
#endif

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Information that are used during enumeration
*/
___WEAK
const USB_DEVICE_INFO USB_DeviceInfo = {
#if 0
   0x085F,      // use with EcoGamma driver on windows
   0x2130,
#else
   0x8765,      // use on linux or with Thesycon driver on windows
   0x3003,
#endif
   "SEGGER",
   "SEGGER ECM device",
   "13245678"
};


//
// USB ECM related variables
//
SECTION_NO_INIT_STATIC("USB_RAM")
U8         _abReceiveBuffer[USB_HS_BULK_MAX_PACKET_SIZE];
static unsigned   _IFaceId;
//
// IP related
//
SECTION_NO_INIT_STATIC("IP_RAM") U32 _aPool[ALLOC_SIZE / 4];  // This is the memory area used by the stack
/*********************************************************************
*
*       Static code
*
**********************************************************************
*/
/*********************************************************************
*
*       _AddECM
*
*  Function description
*    Adds and configures ECM communication device class to USB stack.
*/
static void _AddECM(void) {
  USB_ECM_INIT_DATA InitData;

  memset(&InitData, 0, sizeof(InitData));
  InitData.EPOut      = USBD_AddEP(USB_DIR_OUT, USB_TRANSFER_TYPE_BULK, 0, _abReceiveBuffer, sizeof(_abReceiveBuffer));
  InitData.EPIn       = USBD_AddEP(USB_DIR_IN,  USB_TRANSFER_TYPE_BULK, 0, NULL, 0);
  InitData.EPInt      = USBD_AddEP(USB_DIR_IN,  USB_TRANSFER_TYPE_INT, 32, NULL, 0);
  InitData.pDriverAPI = &USB_Driver_IP_NI;
  InitData.DriverData.pDriverData = (void *)_IFaceId;
  USBD_ECM_Add(&InitData);
}

/*********************************************************************
*
*       _Connect()
*
* Function description
*   Performs a first time initialization for the USB ECM.
*
* Parameters
*   IFaceId: Zero-based interface index.
*
* Return value
*   O.K. :   0.
*   Error: < 0.
*/
static int _Connect(unsigned IFaceId) {
  U32 DNSServer;

  DNSServer = DNS_SERVER_IP_ADDR;
  IP_DHCPS_ConfigPool(IFaceId, DHCPS_START_IP_ADDR, IP_BYTES2ADDR(0xff,0xff,0xff,0xf8), 4);  // Setup IP pool to distribute.
  IP_DHCPS_ConfigDNSAddr(IFaceId, &DNSServer, 1);
  IP_DHCPS_Init(IFaceId);
  IP_DHCPS_Start(IFaceId);
  USB_Init();
  USBD_SetDeviceInfo(&USB_DeviceInfo);
  _AddECM();
//  OS_CREATETASK(&_ECMTCB, "USB ECM Task", _ECMTask, TASK_PRIO_ECM_TASK, _aECMStack);
  USB_Start();
  return 0;                   // Successfully connected.
}

/*********************************************************************
*
*       _Disconnect()
*
* Function description
*   Disconnects from ISP.
*
* Parameters
*   IFaceId: Zero-based interface index.
*
* Return value
*   O.K. :   0.
*   Error: < 0.
*/
static int _Disconnect(unsigned IFaceId) {
  (void)IFaceId;
  return 0;
}


/*********************************************************************
*
*       Public code
*
**********************************************************************
*/


/*********************************************************************
*
*       IP_X_Config
*
*  Function description
*    This function is called by the IP stack during IP_Init().
*
*  Typical memory/buffer configurations:
*    Microcontroller system, size optimized
*      #define ALLOC_SIZE 0x3000                    // 12KBytes RAM
*      mtu = 576;                                   // 576 is minimum acc. to RFC, 1500 is max. for Ethernet
*      IP_SetMTU(0, mtu);                           // Maximum Transmission Unit is 1500 for ethernet by default
*      IP_AddBuffers(8, 256);                       // Small buffers.
*      IP_AddBuffers(4, mtu + 16);                  // Big buffers. Size should be mtu + 16 byte for ethernet header (2 bytes type, 2*6 bytes MAC, 2 bytes padding)
*      IP_ConfTCPSpace(1 * (mtu-40), 1 * (mtu-40)); // Define the TCP Tx and Rx window size
*
*    Microcontroller system, speed optimized or multiple connections
*      #define ALLOC_SIZE 0x6000                    // 24 KBytes RAM
*      mtu = 1500;                                  // 576 is minimum acc. to RFC, 1500 is max. for Ethernet
*      IP_SetMTU(0, mtu);                           // Maximum Transmission Unit is 1500 for ethernet by default
*      IP_AddBuffers(12, 256);                      // Small buffers.
*      IP_AddBuffers(6, mtu + 16);                  // Big buffers. Size should be mtu + 16 byte for ethernet header (2 bytes type, 2*6 bytes MAC, 2 bytes padding)
*      IP_ConfTCPSpace(3 * (mtu-40), 3 * (mtu-40)); // Define the TCP Tx and Rx window size
*
*    System with lots of RAM
*      #define ALLOC_SIZE 0x20000                   // 128 KBytes RAM
*      mtu = 1500;                                  // 576 is minimum acc. to RFC, 1500 is max. for Ethernet
*      IP_SetMTU(0, mtu);                           // Maximum Transmission Unit is 1500 for ethernet by default
*      IP_AddBuffers(50, 256);                      // Small buffers.
*      IP_AddBuffers(50, mtu + 16);                 // Big buffers. Size should be mtu + 16 byte for ethernet header (2 bytes type, 2*6 bytes MAC, 2 bytes padding)
*      IP_ConfTCPSpace(5 * (mtu-40), 5 * (mtu-40)); // Define the TCP Tx and Rx window size
*/
void IP_X_Config(void) {
  int mtu;
  int IFaceId;

  IP_AssignMemory(_aPool, sizeof(_aPool));         // Assigning memory should be the first thing
  IFaceId = IP_AddEtherInterface(DRIVER);          // Add driver for your hardware.
  IP_SetAddrMask(SERVER_IP_ADDR, SUBNET_MASK);     // Assign a local address 10.0.0.1/8
  IP_SetIFaceConnectHook(IFaceId, _Connect);
  IP_SetIFaceDisconnectHook(IFaceId, _Disconnect);
  _IFaceId = IFaceId;
  //
  // Run-time configure buffers.
  // The default setup will do for most cases.
  //
  mtu = 1500;                                  // 576 is minimum acc. to RFC, 1500 is max. for Ethernet
  IP_SetMTU(IFaceId, mtu);                     // Maximum Transmission Unit is 1500 for ethernet by default
  IP_AddBuffers(12, 256);                      // Small buffers.
  IP_AddBuffers( 9, mtu + 16);                 // Big buffers. Size should be mtu + 16 byte for ethernet header (2 bytes type, 2*6 bytes MAC, 2 bytes padding)
  IP_ConfTCPSpace(3 * (mtu-40), 3 * (mtu-40)); // Define the TCP Tx and Rx window size
  IP_SOCKET_SetDefaultOptions(0
//                              | SO_TIMESTAMP   // Send TCP timestamp to optimize the round trip time measurement. Normally not used in LAN.
                              | SO_KEEPALIVE   // Enable keepalives by default for TCP sockets.
                             );
  //
  // Define log and warn filter.
  // Note: The terminal I/O emulation might affect the timing of your
  //       application, since most debuggers need to stop the target
  //       for every terminal I/O output unless you use another
  //       implementation such as DCC or SWO.
  //
  IP_SetWarnFilter(0xFFFFFFFF);              // 0xFFFFFFFF: Do not filter: Output all warnings.
  IP_SetLogFilter(0
                  | IP_MTYPE_APPLICATION     // Output application messages.
                  | IP_MTYPE_INIT            // Output all messages from init.
                  | IP_MTYPE_LINK_CHANGE     // Output a message if link status changes.
                  | IP_MTYPE_PPP             // Output all PPP/PPPoE related messages.
                  | IP_MTYPE_DHCP            // Output general DHCP status messages.
#if IP_SUPPORT_IPV6
                  | IP_MTYPE_IPV6            // Output IPv6 address related messages
#endif
//                  | IP_MTYPE_DHCP_EXT        // Output additional DHCP messages.
//                  | IP_MTYPE_CORE            // Output log messages from core module.
//                  | IP_MTYPE_ALLOC           // Output log messages for memory allocation.
//                  | IP_MTYPE_DRIVER          // Output log messages from driver.
//                  | IP_MTYPE_ARP             // Output log messages from ARP layer.
//                  | IP_MTYPE_IP              // Output log messages from IP layer.
//                  | IP_MTYPE_TCP_CLOSE       // Output a log messages if a TCP connection has been closed.
//                  | IP_MTYPE_TCP_OPEN        // Output a log messages if a TCP connection has been opened.
//                  | IP_MTYPE_TCP_IN          // Output TCP input logs.
//                  | IP_MTYPE_TCP_OUT         // Output TCP output logs.
//                  | IP_MTYPE_TCP_RTT         // Output TCP round trip time (RTT) logs.
//                  | IP_MTYPE_TCP_RXWIN       // Output TCP RX window related log messages.
//                  | IP_MTYPE_TCP             // Output all TCP related log messages.
//                  | IP_MTYPE_UDP_IN          // Output UDP input logs.
//                  | IP_MTYPE_UDP_OUT         // Output UDP output logs.
//                  | IP_MTYPE_UDP             // Output all UDP related messages.
//                  | IP_MTYPE_ICMP            // Output ICMP related log messages.
//                  | IP_MTYPE_NET_IN          // Output network input related messages.
//                  | IP_MTYPE_NET_OUT         // Output network output related messages.
//                  | IP_MTYPE_DNS             // Output all DNS related messages.
//                  | IP_MTYPE_SOCKET_STATE    // Output socket status messages.
//                  | IP_MTYPE_SOCKET_READ     // Output socket read related messages.
//                  | IP_MTYPE_SOCKET_WRITE    // Output socket write related messages.
//                  | IP_MTYPE_SOCKET          // Output all socket related messages.
                 );
  //
  // Add protocols to the stack.
  //
  IP_TCP_Add();
  IP_UDP_Add();
  IP_ICMP_Add();
#if IP_SUPPORT_IPV6
  IP_IPV6_Add(IFaceId);
#endif
}

/*************************** End of file ****************************/
