/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
-------------------------- END-OF-HEADER -----------------------------

File    : BSP_IP.c
Purpose : BSP for embOS/IP on SEGGER emPower V2 with Freescale K66 MCU.
*/

#include "BSP_IP.h"
#include "SEGGER.h"
#include "RTOS.h"     // For OS_Enter/LeaveInterrupt() .

#include "MK66F18.h"  // Device specific header file, contains CMSIS defines.

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

//
// GPIO setup bits.
//
#define PCR_PS_BIT                    (0)
#define PCR_PE_BIT                    (1)
#define PCR_ODE_BIT                   (5)
#define PCR_MUX_BIT                   (8)

//
// Ethernet interrupt IDs as provided by the driver.
//
#define ENET_EIR_TX_INT_ID     76
#define ENET_EIR_RX_INT_ID     77
#define ENET_EIR_ERROR_INT_ID  78

/*********************************************************************
*
*       Types, local
*
**********************************************************************
*/

typedef void ETH_ISR_HANDLER(void);

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static ETH_ISR_HANDLER* _pfEthRxISRHandler;
static ETH_ISR_HANDLER* _pfEthTxISRHandler;
static ETH_ISR_HANDLER* _pfEthErrorISRHandler;

/*********************************************************************
*
*       Prototypes
*
*  Declare ISR handler here to avoid "no prototype" warning.
*  They are not declared in any CMSIS header.
*
**********************************************************************
*/

#ifdef __cplusplus
extern "C" {
#endif

void ENET_Receive_IRQHandler(void);
void ENET_Transmit_IRQHandler(void);
void ENET_Error_IRQHandler(void);

#ifdef __cplusplus
}
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _Init()
*
*  Function description
*    This routine is called from the Ethernet driver to initialize
*    port pins and enable clocks required for Ethernet.
*
*  Parameters
*    IFaceId: Zero-based interface index.
*/
static void _Init(unsigned IFaceId) {
  SEGGER_USE_PARA(IFaceId);

  SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK  // Enable clock for Port A.
            |  SIM_SCGC5_PORTB_MASK  // Enable clock for Port B.
            |  SIM_SCGC5_PORTE_MASK  // Enable clock for Port E.
            ;
  //
  // Configure Ethernet GPIO pins.
  //
  PORTA_PCR12 = 0x00000400;                              // PTA12, RMII0_RXD1/MII0_RXD1
  PORTA_PCR13 = 0x00000400;                              // PTA13, RMII0_RXD0/MII0_RXD0
  PORTA_PCR14 = 0x00000400;                              // PTA14, RMII0_CRS_DV/MII0_RXDV
  PORTA_PCR15 = 0x00000400;                              // PTA15, RMII0_TXEN/MII0_TXEN
  PORTA_PCR16 = 0x00000400;                              // PTA16, RMII0_TXD0/MII0_TXD0
  PORTA_PCR17 = 0x00000400;                              // PTA17, RMII0_TXD1/MII0_TXD1
  PORTB_PCR0 &= ~0x00000700uL;
  PORTB_PCR0 |= ((4uL << PCR_MUX_BIT) |                  // PTB0, RMII0_MDIO/MII0_MDIO
                 (1uL << PCR_PS_BIT)  |                  // Select pullup
                 (1uL << PCR_PE_BIT)) ;                  // Enable pullup
  PORTB_PCR1 &= ~0x00000700uL;
  PORTB_PCR1 |= (4uL << PCR_MUX_BIT);                    // PTB1, RMII0_MDC/MII0_MDC
  //
  // Setup other port pins for Ethernet.
  //
  PORTE_PCR26  =  PORT_PCR_MUX(2)         // Set PTE26 as ENET_1588_CLKIN.
               |  PORT_PCR_ODE_MASK       // Set Open Drain Enable.
               ;
  SIM_SOPT2   |= SIM_SOPT2_RMIISRC_MASK;  // Set RMII clock reference from ENET_1588_CLKIN.
  //
  // PTB2 is used for reset output to the external Ethernet PHY.
  // In order to reset this device, we setup the pin to GPIO output
  // and assert a reset.
  //
  PORTB_PCR2  = 0x0100uL;    // Set PTB2 to GPIO mode.
  GPIOB_PDDR |= (1uL << 2);  // Set PTB2 to output.
  GPIOB_PSOR  = (1uL << 2);  // Release reset for a defined state.
  OS_Delay(2);               // Grant the port pin a short time.
  GPIOB_PCOR  = (1uL << 2);  // Activate the PHY reset (active low).
  //
  // Application Note: Davicom DM9161A / DM9161E - Power on reset
  // The reset pin must be asserted low for at least 10ms.
  //
  OS_Delay(15);
  GPIOB_PSOR = (1uL << 2);  // Release the PHY reset (active low).
}

/*********************************************************************
*
*       _InstallISR()
*
*  Function description
*    Installs the IP stack interrupt handler for Ethernet.
*
*  Parameters
*    IFaceId: Zero-based interface index.
*    pPara  : Parameters for the interrupt handler to install:
*               - pfISR   : Pointer to the interrupt handler of the
*                           IP stack.
*               - ISRIndex: Default interrupt index as far as known by
*                           the driver.
*                           For drivers that fit multiple devices this
*                           might not always be the correct index and
*                           should then be replaced by the correct
*                           index within this function.
*                - Prio   : Suggested priority by the IP stack. This
*                           is only a suggestion and should be
*                           replaced with the desired priority within
*                           this function.
*/
static void _InstallISR(unsigned IFaceId, BSP_IP_INSTALL_ISR_PARA* pPara) {
  int ISRIndex;

  SEGGER_USE_PARA(IFaceId);

  ISRIndex = pPara->ISRIndex;
  MPU_CESR = 0;  // Disable MPU. Allow all accesses from all bus masters.
  switch (ISRIndex) {
  case ENET_EIR_TX_INT_ID:     // Install Tx handler ?
    _pfEthTxISRHandler    = pPara->pfISR;
    break;
  case ENET_EIR_RX_INT_ID:     // Install Rx handler ?
    _pfEthRxISRHandler    = pPara->pfISR;
    break;
  case ENET_EIR_ERROR_INT_ID:  // Install error handler ?
    _pfEthErrorISRHandler = pPara->pfISR;
    break;
  }
  ISRIndex += 7;                                            // Push suggested ISRIndex to the index of K66.
  NVIC_SetPriority((IRQn_Type)ISRIndex, pPara->Prio >> 4);  // The CMSIS NVIC_SetPriority() function expects priority values in the lower bits.
  NVIC_EnableIRQ((IRQn_Type)ISRIndex);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       ENET_Receive_IRQHandler()
*
*  Function description
*    Ethernet Rx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are now in interrupt context.
*/
void ENET_Receive_IRQHandler(void) {
  OS_EnterInterrupt();
  if (_pfEthRxISRHandler) {
    (_pfEthRxISRHandler)();
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       ENET_Transmit_IRQHandler()
*
*  Function description
*    Ethernet Tx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are now in interrupt context.
*/
void ENET_Transmit_IRQHandler(void) {
  OS_EnterInterrupt();
  if (_pfEthTxISRHandler) {
    (_pfEthTxISRHandler)();
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       ENET_Error_IRQHandler()
*
*  Function description
*    Ethernet error interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are now in interrupt context.
*/
void ENET_Error_IRQHandler(void) {
  OS_EnterInterrupt();
  if (_pfEthErrorISRHandler) {
    (_pfEthErrorISRHandler)();
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       Public API structures
*
**********************************************************************
*/

const BSP_IP_API BSP_IP_Api = {
  _Init,       // pfInit
  NULL,        // pfDeInit
  _InstallISR  // pfInstallISR
};

/*************************** End of file ****************************/
