/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File    : USB_Config_SEGGER_emPower.c
Purpose : Config file for SEGGER's emPower eval board.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "USB.h"
#include "BSP_USB.h"
#include "MK66F18.h"

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
#define USBOTG_BASE           0x400A1000
#define USB_DEV_OTGSC         *((volatile U32*)(USBOTG_BASE +0x1A4))

#define USB_ISR_PRIO  (254)

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       _HWAttach
*/
static void _HWAttach(void) {
  //
  // Uncomment the line to enable VBUS charge. This allows to enable the USB PHY
  // without connecting the VBUS pin to the real VBUS of USB.
  //
  USB_DEV_OTGSC |= (1 << 1);
}

/*********************************************************************
*
*       _EnableISR
*/
static void _EnableISR(USB_ISR_HANDLER * pfISRHandler) {
  BSP_USB_InstallISR_Ex((U32)USBHS_IRQn, pfISRHandler, USB_ISR_PRIO);
}

/*********************************************************************
*
*       USBD_X_Config
*
*  Function description
*   Initialize the USB HS controller.
*
*  Notes
*    (1) Change priorities for the flash slave module:
*        Leaving it in the default configuration which is
*           M0 (ARM core code bus)    -> 0 (highest priority)
*           M1 (ARM core system bus)  -> 1
*           M2 (DMA/EzPort)           -> 2
*           M3 (Ethernet)             -> 3
*           M4 (USB FS/LS OTG)        -> 4
*           M5 (SDHC)                 -> 5
*           M6 (USB HS/FS/LS OTG)     -> 6 (lowest priority)
*         access to the flash from the USB FS Controller is stalled
*         which result in sending/retrieving less data than expected
*         which results in data transmissions error (mostly CRC checksum error).
*
*         We will give other DMA masters a higher priority otherwise it is
*         not guaranteed that they will work properly as described above.
*
*           M0 (ARM core code bus)    -> 5
*           M1 (ARM core system bus)  -> 6 (lowest priority)
*           M2 (DMA/EzPort)           -> 4
*           M3 (Ethernet)             -> 3
*           M4 (USB FS/LS OTG)        -> 2
*           M5 (SDHC)                 -> 1
*           M6 (USB HS/FS/LS OTG)     -> 0 (highest priority)
*    (2) The current limit function must be disabled before
*        beginning USB data communication on either of the USB0
*        or USB1 ports, and should remain disabled as long as
*        the USB ports are in use.
*        This is also described in "Mask Set Errata for Mask 0N65N, Rev. 17Nov2015" as e9359:
*          "USBReg: In some conditions when using both VREG_INn inputs, voltage regulator
*                   current limit can fall below specification"
*/
void USBD_X_Config(void) {
  U32 Temp;

#if USBD_SUPPORT_PROFILE
  USBD_SYSVIEW_Init();
#endif
  //
  //  Allow USB controller (Master 6) to access the flash area region (read-only)
  //
  FMC->PFAPR |= FMC_PFAPR_M6AP(1);                              // 0: No access, 1: read-only access, 2: write-only access, 3: r/w access
  //
  // MPU Configuration
  //
  MPU->RGDAAC[0] |= MPU_RGDAAC_M6RE_MASK
                 |  MPU_RGDAAC_M6WE_MASK
                 ;
  AXBS->SLAVE[0].PRS = 0x02103465;                              // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  AXBS->SLAVE[1].PRS = 0x02103465;                              // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  AXBS->SLAVE[2].PRS = 0x02103465;                              // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  AXBS->SLAVE[3].PRS = 0x02103465;                              // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  //
  // Clock configuration
  //
  SIM->USBPHYCTL |= SIM_USBPHYCTL_USBDISILIM_MASK;              // s. Note(2): Disable current limiter for the USB Voltage Regulator.
  SIM->SOPT2 |= SIM_SOPT2_USBREGEN_MASK;                        // Enable USB PHY PLL regulator
  SIM->SCGC3 |= SIM_SCGC3_USBHSPHY_MASK;                        // Open USB PHY clock gate
  SIM->SCGC3 |= SIM_SCGC3_USBHS_MASK;                           // Open USB HS clock gate
  USB_OS_Delay(5);
  Temp = SIM->USBPHYCTL;
  Temp &= ~(SIM_USBPHYCTL_USB3VOUTTRG_MASK);
  Temp |= SIM_USBPHYCTL_USB3VOUTTRG(6);                         // Trim the USB regulator output to be 3.13V
  SIM->USBPHYCTL = Temp;
  SIM->USBPHYCTL &= ~SIM_USBPHYCTL_USBVREGSEL_MASK;
  USBPHY->TRIM_OVERRIDE_EN = 0x001f;
  USBPHY->PLL_SIC_SET = USBPHY_PLL_SIC_PLL_POWER_MASK;
  USBPHY->PLL_SIC_SET = USBPHY_PLL_SIC_PLL_DIV_SEL(2);
  USBPHY->PLL_SIC_CLR = USBPHY_PLL_SIC_PLL_BYPASS_MASK;         // Clear bypass bit
  USBPHY_PLL_SIC_SET  = USBPHY_PLL_SIC_PLL_EN_USB_CLKS_MASK;    // Enable USB clock output from USB PHY PLL
  while ((USBPHY->PLL_SIC & USBPHY_PLL_SIC_PLL_LOCK_MASK) == 0);
  USBPHY->CTRL_CLR  = USBPHY_CTRL_SFTRST_MASK;
  USBPHY->CTRL_CLR  = USBPHY_CTRL_CLKGATE_MASK;
  USBPHY->CTRL_SET  = USBPHY_CTRL_SET_ENUTMILEVEL2_MASK;
  USBPHY->CTRL_SET  = USBPHY_CTRL_SET_ENUTMILEVEL3_MASK;
  USBPHY->PWD = 0;                                              // For normal operation
  USBPHY->ANACTRL_CLR = USBPHY_ANACTRL_DEV_PULLDOWN_MASK;
  USBPHY->ANACTRL_CLR = USBPHY_ANACTRL_PFD_CLKGATE_MASK;
  while ((USBPHY->ANACTRL & USBPHY_ANACTRL_PFD_STABLE_MASK) == 0);
  USBPHY->TX |= 1<<24;
  USB_DRIVER_KinetisEHCI_ConfigAddr(0x400A1000);
  USBD_AddDriver(&USB_Driver_Freescale_KinetisEHCI);
  USBD_SetISRMgmFuncs(_EnableISR, USB_OS_IncDI, USB_OS_DecRI);
  USBD_SetAttachFunc(_HWAttach);
}

/*********************************************************************
*
*       USBD_X_EnableInterrupt
*/
void USBD_X_EnableInterrupt(void) {
  NVIC_EnableIRQ(USBHS_IRQn);
}

/*********************************************************************
*
*       USBD_X_DisableInterrupt
*/
void USBD_X_DisableInterrupt(void) {
  NVIC_DisableIRQ(USBHS_IRQn);
}

/*************************** End of file ****************************/
