/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File    : USBH_Config_KinetisFS_SEGGER_emPower.c
Purpose : emUSB Host configuration file for SEGGER's emPower eval board.
Note    : Do not use the WFI instruction with the USBOTG module.
          See errata e7166 for chip masks 4N96B, 1N96B, 3N96B.
---------------------------END-OF-HEADER------------------------------
*/

/*********************************************************************
*
*       #include Section
*
**********************************************************************
*/
#include <stdlib.h>
#include "USBH.h"
#include "USBH_HW_KinetisFS.h"
#include "BSP_USB.h"
#include "MK66F18.h"

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
#define ALLOC_SIZE               0xC000

/*********************************************************************
*
*       Defines, non-configurable
*
**********************************************************************
*/

#define SIM_SOPT1_USBSTBY_SHIFT       30
#define SIM_SOPT1_USBREGEN_SHIFT      31
#define SIM_SOPT2_PLLFLLSEL_SHIFT     16
#define SIM_SOPT2_USBSRC_SHIFT        18
#define SIM_SCGC4_USBOTG_MASK         0x40000u
#define USB_FRACTIONAL_VALUE          (((0x07 - 1) << 1) | 1)

#define USB_OTG_BASE_ADDR            0x40072000

#define USB_ISR_ID    73
#define USB_ISR_PRIO  250

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static U32 _aPool [((ALLOC_SIZE) / 4)];

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       _InitUSBHw
*
*  Function description
*   Initialize the USB FS OTG controller.
*
*  Notes
*    (1) Change priorities for the flash slave module:
*        Leaving it in the default configuration which is
*           M0 (ARM core code bus)    -> 0 (highest priority)
*           M1 (ARM core system bus)  -> 1
*           M2 (DMA/EzPort)           -> 2
*           M3 (Ethernet)             -> 3
*           M4 (USB FS/LS OTG)        -> 4
*           M5 (SDHC)                 -> 5
*           M6 (USB HS/FS/LS OTG)     -> 6 (lowest priority)
*         access to the flash from the USB FS Controller is stalled
*         which result in sending/retrieving less data than expected
*         which results in data transmissions error (mostly CRC checksum error).
*
*         We will give other DMA masters a higher priority otherwise it is
*         not guaranteed that they will work properly as described above.
*
*           M0 (ARM core code bus)    -> 5
*           M1 (ARM core system bus)  -> 6 (lowest priority)
*           M2 (DMA/EzPort)           -> 4
*           M3 (Ethernet)             -> 3
*           M4 (USB FS/LS OTG)        -> 2
*           M5 (SDHC)                 -> 1
*           M6 (USB HS/FS/LS OTG)     -> 0  (highest priority)
*/
static void _InitUSBHw(void) {
  unsigned int reg;

  SIM->SOPT1 |=  (1UL << SIM_SOPT1_USBREGEN_SHIFT);
  SIM->SOPT1 &= ~(1UL << SIM_SOPT1_USBSTBY_SHIFT);
  //
  //  Allow Busmaster 4 (USB OTG FS controller) to read and write
  //
  MPU->RGDAAC[0] = MPU_RGDAAC_M4RE_MASK
                 | MPU_RGDAAC_M4WE_MASK
                 ;
  //
  //  Allow USB OTG FS controller (Bust Master 4) to access the flash area region (read-only)
  //
  //
  FMC->PFAPR |= FMC_PFAPR_M4AP(1);                  // 0: No access, 1: read-only access, 2: write-only access, 3: r/w access
  //
  // MPU Configuration
  //
  MPU_CESR     =0;                                  // MPU is disable. All accesses from all bus masters are allowed
  //
  //
  AXBS->SLAVE[0].PRS = 0x02103465;                        // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  AXBS->SLAVE[1].PRS = 0x02103465;                        // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  AXBS->SLAVE[2].PRS = 0x02103465;                        // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  AXBS->SLAVE[3].PRS = 0x02103465;                        // s. Note(1): Give the USB OTG FS controller a higher priority than the CPU masters (I/D Bus and system bus)
  //
  // SIM Configuration
  //
  SIM->SOPT2   |= 1 << SIM_SOPT2_PLLFLLSEL_SHIFT;   // Select PLL output
  SIM->SOPT2   |= 1 << SIM_SOPT2_USBSRC_SHIFT;      // PLL/FLL selected as CLK source
  SIM->CLKDIV2  = USB_FRACTIONAL_VALUE;             // USB Freq Divider
  SIM->SCGC4   |= SIM_SCGC4_USBOTG_MASK;            // USB Clock Gating
  //
  // Enable port power
  //
  reg           = SIM_SCGC5;
  reg          &= ~SIM_SCGC5_PORTE_MASK;
  reg          |= (1uL << SIM_SCGC5_PORTE_SHIFT);
  SIM_SCGC5     = reg;                              // Slow slew rate
  reg           = PORTE_PCR28;
  reg          |= PORT_PCR_SRE_MASK                 // Slow slew rate
               |  PORT_PCR_ODE_MASK                 // Open Drain Enable
               |  PORT_PCR_DSE_MASK;                // High drive strength
  PORTE_PCR28   = reg;
  PORTE_PCR28  |= PORT_PCR_MUX(1);                  // Alternative 1 (GPIO)
  GPIOE_PCOR    = (1 << 28);                        // Clear the port to enable power
  GPIOE_PDDR   |= (1 << 28);                        // Set the port direction to output
}

/*********************************************************************
*
*       _ISR
*
*  Function description
*/
static void _ISR(void) {
  USBH_ServiceISR(0);
}

/*********************************************************************
*
*       USBH_X_Config
*
*  Function description
*/
void USBH_X_Config(void) {
  USBH_AssignMemory((void *)&_aPool[0], ALLOC_SIZE);    // Assigning memory should be the first thing
  // USBH_ConfigSupportExternalHubs (0);           // Default values: The hub module is disabled, this is done to save memory.
  // USBH_ConfigPowerOnGoodTime     (300);         // Default values: 300 ms wait time before the hosts starts communicating with the device.
  //
  // Define log and warn filter
  // Note: The terminal I/O emulation affects the timing
  // of your communication, since the debugger stops the target
  // for every terminal I/O output unless you use RTT!
  //
  USBH_SetWarnFilter(0xFFFFFFFF);               // 0xFFFFFFFF: Do not filter: Output all warnings.
  USBH_SetLogFilter(0
                    | USBH_MTYPE_INIT
                    | USBH_MTYPE_APPLICATION
                    );
  _InitUSBHw();
  USBH_KINETIS_FS_Add((void*)(USB_OTG_BASE_ADDR));
  BSP_USBH_InstallISR_Ex(USB0_IRQn, _ISR, USB_ISR_PRIO);
}
/*************************** End of file ****************************/
